//
// IERI key-manager-lite v0.0.1.1
// Copyright (C) 2000 Eddy L O Jansson & The IEFFHP.
//
// http://hem.passagen.se/eddy1/index.html
// http://www.ugcs.caltech.edu/~jedwin/baldur.html
//
// This file is part of GNU program
//
// GNU program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// See http:/www.gnu.org/ for more information.
//

#include "include/incs.hh"

// Hashtable definition for resource entries
typedef pair<const string, t_key_v1_resentry> res_pair;
typedef multimap<string, t_key_v1_resentry>   tmap_res_entries;

class ie_res_key
{
  public:
   ie_res_key(const string filename);
   ~ie_res_key();

  // private:
   t_key_v1_header            header;
   vector<t_key_v1_bifentry>  bif_entries;
   vector<string>             bif_filenames;
   tmap_res_entries           res_entries;
};

ie_res_key::ie_res_key(const string filename)
{
  fstream  fs;

  header.bif_count=0; // lame error-checking.

  fs.open(filename.c_str(), ios::binary | ios::in);
  if(fs.is_open())
  {
    fs.read(&header,sizeof(header));
    if(strncmp(header.signature,"KEY V1  ",8) == 0)
    {
      // Read BIF entries
      {
	    // cout << "Reading BIF entries... ";
        fs.seekg(header.bif_offset, ios::beg);
        t_key_v1_bifentry an_entry;
        for(ulong32 i=0 ; i < header.bif_count ; i++)
        {
          fs.read(&an_entry, sizeof(an_entry));
	      bif_entries.push_back(an_entry);
        }
      }

      // Read BIF filenames
      {
        fs.seekg(bif_entries[0].bif_filename_offset, ios::beg);
        char an_entry[256];
        for(ulong32 i=0 ; i < header.bif_count ; i++)
        {
          fs.read(&an_entry, bif_entries[i].bif_filename_length);
	      for(ulong32 j=0 ; j<bif_entries[i].bif_filename_length ; ++j) // fix path slashes.
            if(an_entry[j]=='\\') an_entry[j]='/';
	      bif_filenames.push_back(an_entry);
        }
      }

      // Read RES entries
      {
        fs.seekg(header.res_offset, ios::beg);
	    t_key_v1_resentry an_entry;
	    char dummy[9];
        dummy[8]=0; // terminate
        for(ulong32 i=0 ; i < header.res_count ; i++)
	    {
          fs.read(&an_entry, sizeof(an_entry));
          strncpy(dummy, an_entry.id, 8);
	      res_entries.insert(res_pair(dummy, an_entry));
	    }
      }
    } else {
      cerr << "\n[key] File \"" << filename << "\" is not a valid V1 KEY resource." << endl;
    }
    fs.close();

  } else {
    cerr << "\n[key] Couldn't open KEY \"" << filename << "\"" << endl;
  }

};

ie_res_key::~ie_res_key()
{
};