//
// IERI structures ("KEY" and miscellaneous) v0.0.1
// Copyright (C) 2000 Eddy L O Jansson & The IEFFHP.
//
// http://hem.passagen.se/eddy1/index.html
// http://www.ugcs.caltech.edu/~jedwin/baldur.html
//
// This file is part of GNU program
//
// GNU program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// See http:/www.gnu.org/ for more information.
//

// (- Miscellaneous ----------------------------------------------------------)

 static const uword16 RES_TYPE_BAM  = 0x3e8;
 static const uword16 RES_TYPE_WED  = 0x3e9;
 static const uword16 RES_TYPE_CHUI = 0x3ea;
 static const uword16 RES_TYPE_TIS  = 0x3eb;
 static const uword16 RES_TYPE_MOS  = 0x3ec;
 static const uword16 RES_TYPE_ITM  = 0x3ed;

 static const uword16 RES_TYPE_BCS  = 0x3ef;
 static const uword16 RES_TYPE_IDS  = 0x3f0;
 static const uword16 RES_TYPE_CRE  = 0x3f1;
 static const uword16 RES_TYPE_AREA = 0x3f2;
 static const uword16 RES_TYPE_DLG  = 0x3f3;
 static const uword16 RES_TYPE_2DA  = 0x3f4;
 static const uword16 RES_TYPE_GAME = 0x3f5;
 static const uword16 RES_TYPE_STOR = 0x3f6;
 static const uword16 RES_TYPE_WMAP = 0x3f7;
 static const uword16 RES_TYPE_EFF  = 0x3f8;  // _CHR too -- not used.
 static const uword16 RES_TYPE_BS   = 0x3f9;

 static const uword16 RES_TYPE_INI  = 0x802;  // These from PS:Torment
 static const uword16 RES_TYPE_SRC  = 0x803;

 static const uword16 filetype_id[] = {
  0x0000, 0x0001, 0x0002, 0x0004, 0x03e8, 0x03e9, 0x03ea, 0x03eb, 0x03ec,
  0x03ed, 0x03ee, 0x03ef, 0x03f0, 0x03f1, 0x03f2, 0x03f3, 0x03f4, 0x03f5,
  0x03f6, 0x03f7, 0x03f8, 0x03f9, 0x0802, 0x0803};

 static const char* filetype_ext[] = {
  "",     "bmp",  "mve",  "wav",  "bam",  "wed",  "chu",  "tis",  "mos",
  "itm",  "spl",  "bcs",  "ids",  "cre",  "are",  "dlg",  "2da",  "gam",
  "sto",  "wmp",  "eff",  "bs",   "ini",  "src"};

 static const char* filetype_desc[] = {
  "n/a",
  "Bitmap images",
  "Movies",
  "Wave sound files",
  "Bitmap animations",
  "Area information",
  "Interface definitions",
  "Tileset graphics",
  "Tiled bitmapped images",
  "Items",
  "Spells",
  "Compiled scripts",
  "Identifiers",
  "Creatures",
  "Areas",
  "Dialogue trees",
  "Two dimentional array",
  "Game structures",
  "Stores",
  "World maps",
  "Effects",
  "Scripts",
  "Configuration files",
  "String references"
 };

 typedef ulong32 strref;
 typedef char resref[8];

// (- KEY --------------------------------------------------------------------)

typedef struct t_rt_key_v1_header t_key_v1_header;
struct t_rt_key_v1_header
{
  char    signature[8];            // 'KEY V1  '
  ulong32 bif_count;
  ulong32 res_count;
  ulong32 bif_offset;
  ulong32 res_offset;
} __attribute__ ((packed));

typedef struct t_rt_key_v1_bifentry t_key_v1_bifentry;
struct t_rt_key_v1_bifentry
{
  ulong32 bif_size;
  ulong32 bif_filename_offset;
  uword16 bif_filename_length;
  uword16 search_override:1;
  uword16 search_cache:1;        // used in-mem by the Infinity Engine(TM)
  uword16 on_disc1:1;
  uword16 on_disc2:1;
  uword16 on_disc3:1;
  uword16 on_disc4:1;
  uword16 on_disc5:1;
  uword16 on_disc6:1;
  uword16 on_disc7:1;
  uword16 unknown:7;
} __attribute__ ((packed));

typedef struct t_rt_key_v1_resentry t_key_v1_resentry;
struct t_rt_key_v1_resentry
{
  resref  id;
  uword16 type;
  union{
    ulong32 raw;
    struct{
     ulong32 residx:12;       // 14 ?
     ulong32 tilesetidx:8;    //  6 ?
     ulong32 bifidx:12;
    } bits;
  } loc;
} __attribute__ ((packed));