//
// IEFFHP keytool v0.1.1
// Copyright (C) 2000 Eddy L O Jansson & The IEFFHP.
//
// http://hem.passagen.se/eddy1/index.html
// http://www.ugcs.caltech.edu/~jedwin/baldur.html
//
// This file is part of GNU program
//
// GNU program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// See http:/www.gnu.org/ for more information.
//

#include "include/incs.hh"
#include "ieri_key-lite.cc"

string get_filetype_ext(const uword16 resid)
{
  for(unsigned int i=0;i<sizeof(filetype_id)/sizeof(filetype_id[0]);i++)
  {
    if(resid==filetype_id[i]) return string(filetype_ext[i]);
  }
  return string("");
}

string get_filetype_desc(const uword16 resid)
{
  for(unsigned int i=0;i<sizeof(filetype_id)/sizeof(filetype_id[0]);i++)
  {
    if(resid==filetype_id[i]) return string(filetype_desc[i]);
  }
  return string("");
}

void html_output_header(ofstream& f, const string title)
{
  f << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">" << endl;
  f << "<html>\n <head>" << endl;
  f << "  <title>" << title << "</title>" << endl;
  f << "  <link rel=\"stylesheet\" href=\"ieffhp.css\" type=\"text/css\">" << endl;
  f << " </head>\n\n";
}

void html_output_footer(ofstream& f)
{
  f << "<p class=\"disclaimer\">All trademarks acknowledged. Visit the <a href=\"http://www.ugcs.caltech.edu/~jedwin/baldur.html\">IEFFHP</a></p>\n";
}


int main(int argc, char* argv[])
{
  const string version = "0.1.1";
  string filename="", s;
  --argc;

  if(argc>0) filename = string(argv[1]);

  if(filename=="--version")
  {
    cout << "keytool (ieffhp tools) v" << version << endl;
    cout << "Copyright (C)2000 Eddy L O Jansson (srm_dfr@hotmail.com) & The IEFFHP\n" << endl;
    cout << "Keytool comes with NO WARRANTY, to the extent permitted by law." << endl;
    cout << "You may redistribute copies of Keytool under the terms of the GNU General Public License." << endl;
    cout << "For more information about these matters, see the file named COPYING." << endl;
    exit(1);
   }

  if((argc==0) || (filename=="--help"))
  {
    cout << "ieffhp tools: keytool v" << version << endl;
    cout << endl;
    cout << "Usage: " << argv[0] << " <filename>" << endl << endl;
    cout << "Generates a HTML-index from an Infinity Engine chitin.key file." << endl;
    cout << endl;
    cout << "Report bugs to <srm_dfr@hotmail.com>" << endl;
    exit(1);
  }

  cout << "Reading and indexing " << filename;
  ie_res_key* res = new ie_res_key(filename);

  if(res->header.bif_count == 0) // sorry, no exceptions in lite-version of res_key.
  {
    cout << "\nerror: Indexing failed." << endl;
    exit(1);
  }

  ofstream html("index.html");
  html.setf(ios::left, ios::adjustfield);
  html.setf(ios::showpoint);
  html.precision(3);

  cout << endl;
  cout << "HTML generation initialized.";

  html_output_header(html,"IEFFHP tools: keytool "+version+" - analysis of file "+filename);
  html << "<body>\n";
  html << "<h1>IEFFHP Keytool " << version << "</h1>\n\n";

  html << "<p>Results of scanning <em>" << filename << "</em></p>\n\n";

  // output TKeyHeader table
  html << "<table class=\"tbl1\" width=\"40%\">\n";
  html << "<colgroup>\n";
  html << " <col width=\"*\">\n";
  html << " <col width=\"12%\">\n";
  html << "</colgroup>\n <thead>\n";
  html << " <tr><th colspan=\"2\">TKeyHeader</th></tr>\n";
  html << "</thead>\n <tbody>\n";
  html << " <tr class=\"o2\"><td>BIF entries</td><td>" << dec << res->header.bif_count  << "</td></tr>\n";
  html << " <tr class=\"e2\"><td>BIF offset</td><td>0x" << hex << res->header.bif_offset << "</td></tr>\n";
  html << " <tr class=\"o2\"><td>RES entries</td><td>" << dec << res->header.res_count  << "</td></tr>\n";
  html << " <tr class=\"e2\"><td>RES offset</td><td>0x" << hex << res->header.res_offset << dec << "</td></tr>\n";
  html << " </tbody>\n</table>\n";

  html << "\n<p>&nbsp;</p>\n";

  // Output RES entries by type table
  html << "<table class=\"tbl1\" width=\"80%\">\n";
  html << "<colgroup>\n";
  html << " <col width=\"5%\" class=\"coltype\">\n";
  html << " <col width=\"*\" class=\"coldesc\">\n";
  html << " <col width=\"5%\" class=\"colext\">\n";
  html << " <col width=\"8%\" class=\"colcnt\">\n";
  html << " <col width=\"12%\" class=\"colperc\">\n";
  html << "</colgroup>\n <thead>\n";
  html << " <tr><th colspan=\"5\">RES Entries by type</th></tr>\n";
  html << " <tr><th>Type</th><th>Description</th><th>Ext</th><th>Count</th><th>% of total count</th></tr>\n";
  html << "</thead>\n <tbody>\n";

  // Generate main table over used resource types
  map<long, uword16> used_types;
  {
    tmap_res_entries::iterator it;
    for(it = res->res_entries.begin(); it != res->res_entries.end(); ++it)
    {
     used_types[ (*it).second.type ]++; // can be uninitialized or?!?!
    }
  }
  map<long, uword16>::iterator setit;
  int i=0;
  for(setit = used_types.begin(); setit != used_types.end(); ++setit)
  {
    uword16 thistype = (*setit).first;
    if(i&1){ s="e2"; } else { s="o2"; }
    html << " <tr class=\"" << s << "\">";
    html << "<td>0x" << hex << thistype << "</td>";
    html << "<td><a href=\"keytool_" << get_filetype_ext(thistype) << ".html\">" << get_filetype_desc(thistype) << "</a></td>";
    html << "<td>" << get_filetype_ext(thistype) <<"</td>";
	html << "<td>" << dec << (*setit).second << "</td>";
    html << "<td>" << ( float((*setit).second) / float(res->header.res_count))*100 << "</td>";

    html << "</tr>\n";
    ++i;
  }
  html << " </tbody>\n</table>\n";

  html << "\n<p>&nbsp;</p>\n";

  // calculate total size of bif-files
  ulong32 total_bifsize = 0;
  {
    vector<t_key_v1_bifentry>::iterator it;
    for(it = res->bif_entries.begin(); it != res->bif_entries.end(); it++)
      total_bifsize+= (*it).bif_size / 1024;
  }

  // output TBifEntries table
  html << "<table class=\"tblbif\" width=\"80%\">\n";
  html << "<colgroup>\n";
  html << " <col width=\"5%\" class=\"colid\">\n";
  html << " <col width=\"*\" class=\"colfn\">\n";
  html << " <col width=\"12%\" class=\"colfs\">\n";
  html << " <col width=\"5%\" class=\"coldisk\">\n";
  html << " <col width=\"5%\" class=\"coldisk\">\n";
  html << " <col width=\"5%\" class=\"coldisk\">\n";
  html << " <col width=\"5%\" class=\"coldisk\">\n";
  html << " <col width=\"5%\" class=\"coldisk\">\n";
  html << " <col width=\"5%\" class=\"coldisk\">\n";
  html << " <col width=\"5%\" class=\"coldisk\">\n";
  html << " <col width=\"12%\" class=\"colperc\">\n";
  html << "</colgroup>\n <thead>\n";
  html << " <tr><th colspan=\"11\">BIF Entries</th></tr>\n";
  html << " <tr><th>ID</th><th>Filename</th><th>Filesize</th>";
  html << "<th>Disk 1</th><th>Disk 2</th><th>Disk 3</th><th>Disk 4</th><th>Disk 5</th><th>Disk 6</th><th>Disk 7</th>";
  html << "<th>% of total size</th></tr>\n";
  html << " </thead>\n";
  html << " <tbody>\n";
  for(unsigned int i=0 ; i<res->bif_entries.size() ; i++)
  {
    if(i&1){ s="e2"; } else { s="o2"; }
    html << " <tr class=\"" << s << "\">";
    html << "<td><a name=\"BIF" << dec << i << "\">" << i << "</a></td>";
    html << "<td>" << res->bif_filenames[i] << "</td>";
    html << "<td>" << res->bif_entries[i].bif_size << "</td>";

    char carr[7];
    for(unsigned int j=0;j<sizeof(carr);++j) carr[j]='\n';

    if(res->bif_entries[i].on_disc1) carr[0]='1';
    if(res->bif_entries[i].on_disc2) carr[1]='2';
    if(res->bif_entries[i].on_disc3) carr[2]='3';
    if(res->bif_entries[i].on_disc4) carr[3]='4';
    if(res->bif_entries[i].on_disc5) carr[4]='5';
    if(res->bif_entries[i].on_disc6) carr[5]='6';
    if(res->bif_entries[i].on_disc7) carr[6]='7';
    for(unsigned int j=0;j<sizeof(carr);++j)
      html << "<td>" << carr[j] << "</td>";

    html << "<td>" << (float(res->bif_entries[i].bif_size/1024) / float(total_bifsize))*100 << "</td>";
    html << "</tr>\n";
  }
  html << " <tr class=\"sum\"><td></td><td></td><td>" << total_bifsize << "Kb</td><td></td><td></td><td></td><td></td><td></td><td></td><td></td><td>100%</td></tr>\n";
  html << " </tbody>\n";
  html << "</table>\n";
  html_output_footer(html);
  html << "</body></html>\n";
  html.close();

  // Generate HTML files containing resource information.
  for(setit = used_types.begin(); setit != used_types.end(); ++setit)
  {
    uword16 thistype = (*setit).first;
    string hfn = "keytool_"+get_filetype_ext(thistype)+".html";
    ofstream html(hfn.c_str());
    html_output_header(html,"IEFFHP tools: keytool "+version+" - analysis of file "+filename);

    html << "<table class=\"tbl1\" width=\"40%\">\n";
    html << " <tr><th colspan=\"4\">RES Entries of type 0x" << hex << thistype << "(" << get_filetype_ext(thistype) << ")</th></tr>\n";
    html << " <tr><th>ResID</th><th>ResIndex</th><th>TilesetIndex</th><th>BIFIndex</th></tr>\n";

    int i=0;
    char dummy[9];
    dummy[8]=0; // terminate
    tmap_res_entries::iterator it;
    for(it = res->res_entries.begin(); it != res->res_entries.end(); ++it)
    {
      if((*it).second.type==thistype)
      {
        strncpy(dummy, (*it).second.id, 8);
        if(i&1){ s="e2"; } else { s="o2"; }
        html << " <tr class=\"" << s << "\">";
        html << "<td><a href=\"./cgi-bin/dumpres?id=" << dummy << "&amp;idx=" << (*it).second.loc.bits.residx << "\">" << dummy << "</a></td>";
        html << "<td>" << dec << (*it).second.loc.bits.residx << "</td>";
        html << "<td>" << (*it).second.loc.bits.tilesetidx << "</td>";
        html << "<td><a href=\"index.html#BIF" << (*it).second.loc.bits.bifidx << "\">" << (*it).second.loc.bits.bifidx << "</a></td>";
        html << "</tr>\n";
        ++i;
      }
    }

    html << "</table>\n";

    html_output_footer(html);

    html << "</body><html>\n";
    html.close();
  }

  cout << endl << "HTML generation completed." << endl;
  delete res;
  return 0;
}
